/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.wizards;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

import benten.core.BentenProjectNature;
import benten.twa.BentenTwaConstants;

/**
 * 翻訳プロジェクト作成ウィザードです。
 */
public class NewProjectCreationWizard extends Wizard implements INewWizard, IExecutableExtension {

	private WizardNewProjectCreationPage page;

	public NewProjectCreationWizard() {
		setWindowTitle("翻訳プロジェクトの作成");
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
	}

	public void setInitializationData(IConfigurationElement config, String propertyName, Object data)
			throws CoreException {
	}

	@Override
	public void addPages() {
		page = new WizardNewProjectCreationPage("page"); //$NON-NLS-1$
		page.setTitle("翻訳プロジェクトの作成");
		page.setDescription("翻訳プロジェクトを新規作成します。翻訳対象ごとに、翻訳プロジェクトを 1 つ作成して利用してください。");
		addPage(page);
	}

	@Override
	public boolean performFinish() {
		final IProject project = page.getProjectHandle();
		try {
			project.create(null);
			project.open(null);

			// 翻訳プロジェクト・ネーチャーの追加
			final IProjectDescription description = project.getDescription();
			if (!description.hasNature(BentenProjectNature.NATURE_ID)) {
				final String[] ids = description.getNatureIds();
				final String[] newIds = new String[ids.length + 1];
				System.arraycopy(ids, 0, newIds, 0, ids.length);
				newIds[ids.length] = BentenProjectNature.NATURE_ID;
				description.setNatureIds(newIds);
				project.setDescription(description, null);
			}

			// インポートされた HTML が格納されるデフォルト・ディレクトリは [/.benten/source] とします。
			// このディレクトリには、現在は html しか格納されませんが、
			// 将来は docx や txt があわせて入ることを想定しています。
			createFolder(project, BentenTwaConstants.DEFAULT_DIR);
			createFolder(project, BentenTwaConstants.DEFAULT_SOURCE_DIR);

			createFolder(project, BentenTwaConstants.DEFAULT_XLIFF_DIR);
			createFolder(project, BentenTwaConstants.DEFAULT_TMX_DIR);
			createFolder(project, BentenTwaConstants.DEFAULT_XLIFF_FRAGMENT_DIR);
			createFolder(project, BentenTwaConstants.DEFAULT_TMX_FRAGMENT_DIR);

			project.refreshLocal(IResource.DEPTH_INFINITE, null);
			showPerspective();

		} catch (final CoreException e) {
			throw new IllegalStateException(e);
		}
		return true;
	}

	private void createFolder(IProject project, String name) throws CoreException {
		final IFolder folder = project.getFolder(new Path(name));
		if (!folder.exists()) {
			folder.create(true, true, null);
		}
	}

	private void showPerspective() {
		final IWorkbench workbench = PlatformUI.getWorkbench();
		final IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		final IWorkbenchPage page = window.getActivePage();
		IAdaptable input;
		if (page != null)
			input = page.getInput();
		else
			input = ResourcesPlugin.getWorkspace().getRoot();
		try {
			workbench.showPerspective("benten.translationPerspective", window, input); //$NON-NLS-1$
		} catch (final WorkbenchException e) {
			throw new IllegalStateException(e);
		}
	}
}
